/*
 *   Common definitions file
 *
 *   (c) 2003 by Eduardo Enrique Gonzlez Rodrguez
 *   Distributed under GPL, see license.txt file in the package
 */

#ifndef INCLUDE_DEFINITIONS

#define INCLUDE_DEFINITIONS

/* Limits and sizes */
#define STATICW_BITS 15 /* Num bits of the static window */
#define STATICW_SIZE (1 << STATICW_BITS) /* Static window size */
#define SLIDINGW_BITS 15 /* Num bits of the sliding window */
#define SLIDINGW_SIZE ((1 << SLIDINGW_BITS) - 1) /* Sliding window size */
#define SLIDINGW_MASK SLIDINGW_SIZE
#define BR_TABLE_SIZE (STATICW_SIZE + 1) /* Table bits_for_range */
#define SYMBOL_BITS 8
#define MAX_SYMBOLS (1 << SYMBOL_BITS)
#define SYMBOL_MASK (MAX_SYMBOLS - 1)
#define NBITS_BITS 4
#define NBITS_MASK ((1 << NBITS_BITS) - 1)

/* used by static tables: number of bits, sizes and masks */
#define CODE_BITS1 12
#define TABLE_BITS1 CODE_BITS1
#define CODE_RANGE1 (1 << CODE_BITS1)
#define TABLE_RANGE1 CODE_RANGE1 
#define CODE_MASK1 (CODE_RANGE1 - 1)
#define TABLE_MASK1 CODE_MASK1 
#define CODE_BITS2 (STATICW_BITS - CODE_BITS1)
#define TABLE_BITS2 8
#define CODE_RANGE2 (1 << CODE_BITS2)
#define TABLE_RANGE2 (1 << TABLE_BITS2)
#define CODE_MASK2 (CODE_RANGE2 - 1)
#define TABLE_MASK2 (TABLE_RANGE2 - 1)

/* Maximum value a frequency can take */
#define MAX_TOT_FREQ1 (CODE_RANGE1)
#define MAX_TOT_FREQ2 MAX_SYMBOLS

/* Number of higher frequencies taken for encoding static method */
#define NUM_HIGH_FREQS 30

/* ------------------------------- Macros ------------------------------- */
#define MIN(a, b) ((a < b)? a : b)
/* Floor rounded base-2 logarithm */
#define FRLOG2(X) ((X) & (255 << 8)? (bits_for_range[(X) >> 8] + 8):\
bits_for_range[X])

/* ------------------------------ Typedefs ------------------------------- */

typedef unsigned int dword;

/*
 *   This type is used to address information in the history buffer, or in
 * data structures of its same size or less. Being this buffer no more than
 * 64 Kb in most cases, it is very reasonable to choose a 16 bit data type.
 */
typedef unsigned short int word;

/*
 *   This type is used to accommodate input symbols. We will suppose they are
 * all bytes.
 */
typedef unsigned char symbol;

/*
 *   Type of the data stream to decode from/ encode to.
 */

typedef unsigned int stream;
#define STREAM_MASK (-1L)

#endif
