/*
 *
 *   (c) 2003 by Eduardo Enrique Gonzlez Rodrguez
 *   Distributed under GPL, see license.txt file in the package
 *
 *   File out.inc
 *   Output functions to write to a bit stream:
 *      - streams of bits: write_bits()
 *      - integers in a range: write_number()
 */


/* Writes bits following method 3 (LIFO without bit counter) */
static void
write_bits(stream *output, dword *index, unsigned int to_write,
	   int nbits_to_write)
{
	unsigned int old;

	old = output[*index];
	output[*index] = (output[*index] << nbits_to_write) | to_write;
	if ((-1U >> nbits_to_write) < old) {
		output[*index - 1] = old >>
			(8 * sizeof(output[0]) - nbits_to_write);
		(*index)--;
	}
	return;
}

/*
 *   Writes an integer comprised in the range [0, range) to the output,
 * using phasing in codes.
 *
 *   Let range  =  2^k + b. 0 <= b < 2^k.
 */

static void
write_number_pic(stream *output, dword *index, int number, int range)
{
	unsigned char k;
	int c;

	range++;
	k = FRLOG2(range);
	c = (1 << (k + 1)) - range; /* c = 2^k - b */
	if (number < c)
		write_bits(output, index, number, k);
	else {
		write_bits(output, index, (number - c) & 1, 1);
		write_bits(output, index, (number + c) >> 1, k);
	}
	return;
}

/*
 *   Writes an integer comprised in the range [0, range) to the input.
 *
 *   The integer is encoded arithmetically. The algorithm is:
 *
 *   - Read record pointed by index
 *   - if record * range fits in the record
 *           record = record * range + number
 *     Else
 *           accumulator = record combined with next record
 *           accumulator = accumulator * range + number
 *           decrease index
 */

static void
write_number_ac(stream *output, dword *index, int number, int range)
{
	unsigned long long int acc;

	acc = output[*index];
	acc = acc * range + number;
	output[*index] = acc & STREAM_MASK;
	acc >>= (sizeof (stream) * 8);
	output[*index - 1] = acc;
	*index -= (acc? 1: 0);
	return;
}

