/*
 *   This file contains benchmarking functions and others for the evaluation
 * of the rf coding implementation.
 */


#include <time.h>
#include <math.h>


/*
 *   Prints elapsed time in microseconds.
 *
 *   Usage:
 *
 *   At the beggining of the function:
 *
 *   struct timeval tstart, tend;
 *
 *   gettimeofday(&tstart, NULL);
 *
 *   ...............
 *
 *   gettimeofday(&tend, NULL);
 *   print_elapsed("sentence", tstart, tend);
 *
 *   To use it, time.h must be included
 */

static void
print_elapsed(char *action, struct timeval tstart, struct timeval tend)
{
	long int tot_microsec1;
	long int tot_microsec2;

	tot_microsec1 = 1000000 * (tstart).tv_sec + (tstart).tv_usec;
	tot_microsec2 = 1000000 * (tend).tv_sec + (tend).tv_usec;
	printf ("Elapsed time %s: %ld sec, %ld microsec\n", action,
		(long int) ((tot_microsec2 - tot_microsec1) / 1000000),
		(long int) ((tot_microsec2 - tot_microsec1)) % 1000000);
	return;
}

/*
 *   Tests consistency of the data structures during dynamic encoding.
 */

static void
testall(symbol *input, int input_size, word *freq1, word *address_index1,
	word *address1, word *freq2, word *address_index2, word *address2,
	int pos, int offset, int flag)
{
	int i;
	int s;
	int f[256];

	s = input[pos];
	memset(f, 0, 1024);
	for (i = 0; i < pos; i++)
		f[*(input + i)]++;
	for (i = pos; i < 4095; i++)
		f[*(input + i + offset)]++;

	for (s = 0; s < 256; s++)
		if (((int)freq1[s] + (int)freq2[s]) != f[s])
			printf("#%d Freq mismatch: %d: real: %d, t: %d, it: %d\n",
				flag, (int)s, (int)f[s],
				(int) (freq1[s] + freq2[s]), pos);
	for (s = 0; s < 256; s++)
		for (i = 0; i < freq1[s]; i++) {
			if (input[address1[address_index1[s] - i]+offset] != s)
				printf("Address mismatch: %d, freq1: %d, it:%d\n",
					(int)s, (int)freq1[s], pos);
		}
	for (s = 0; s < 256; s++)
		for (i = 0; i < freq2[s]; i++) {
			if (input[address2[address_index2[s] - i]] != s)
				printf("Address mismatch: %d, freq2: %d, it:%d\n",
					(int)s, (int)freq2[s], pos);
		}
	return;
}

/*
 *  For a different ranges up to a number, print the average efficiency of
 * phasing in coding for that range. Average efficiency = log2(range) /
 * (average bits required to represent number with phasing in codes)
 *
 *  Must compile with the -lm option to work and have bits_for_range
 * initialized prior to issuing calls to phasing_in_eff()
 */

void
phasing_in_eff(void)
{
	int range;
	int num;
	int k;
	int tot;

	for (range = 2; range < 512; range++) {
		tot = 0;
		for (num = 0; num < range; num++) {
			k = FRLOG2(range-1);
			k += (((num & bit_mask[k]) | (1 << k)) < range)? 1: 0;
			tot += k;
		}
		printf("%d %f\n", range,
			(float)(log(range)/log(2)) / (((double)tot) / range));
	}
	exit(0);
	return;
}


